(* Blue Coconut
	Download tracks from iTunes shared libraries.
  Uses AppleScript and Perl

	Copyright (c) 2004 Paul Mison
*)

property pid : 0
property ready : false
property queue : {}

property playlistName : "Blue Coconut"

-- on run handler, for iTunes script menu

on run
	tell application "Finder" to set theApp to (home as string) & "Library:iTunes:Scripts:Blue Coconut.app"
	try
		tell application theApp
			activate
		end tell
	on error
		tell me
			activate
		end tell
	end try
end run

-- UI event handlers

on launched theObject
	if not checkVersion() then quit
	
	-- switch off interface while we wait to poll iTunes and for the Perl
	-- daemon to start
	setStatus("Please wait...")
	disableGet(false, false)
	
	-- shamelessly taken from TN2065
	-- http://developer.apple.com/technotes/tn2002/tn2065.html
	set f to resource path of main bundle
	set ScriptPath to quoted form of POSIX path of POSIX file (f)
	
	set logFile to POSIX path of getLogFilePath()
	do shell script ScriptPath & "/BlueCoconut.pl > " & logFile & " 2>&1 & echo $!"
	set pid to the result
end launched

on will quit theObject
	-- tidy up by killing the Perl process
	if pid > 0 then
		set ps to do shell script ("ps aucx")
		if ps contains pid and ps contains "perl" then
			do shell script "kill " & pid
		end if
	end if
end will quit

on clicked theObject
	if the name of theObject is "Get" then
		getFiles("playing")
	end if
end clicked

on choose menu item theObject
	set menuName to the name of theObject
	
	if menuName = "Download" then
		getFiles("playing")
	else if menuName = "DownloadSelection" then
		getFiles("selection")
	else if menuName = "reveal_iTunes" then
		reveal_iTunes()
	else if menuName = "reveal_Finder" then
		reveal_Finder()
	else if menuName = "showMainWindow" then
		showMainWindow()
	else if menuName = "showHTML" then
		showHTML()
	end if
end choose menu item

on idle
	if not ready then
		getReady()
	end if
	
	if ready and queue is not {} then
		getQueue()
	end if
	
	getiTunesStatus()
	return 2 -- TODO preference
end idle

on activated theObject
	showMainWindow()
end activated

on quit
	continue quit
end quit

-- main functions
on checkVersion()
	set reqVers to "10.3"
	
	-- check Finder. TODO: check iTunes too
	tell application "Finder"
		set theVersion to (get the version) as string
	end tell
	
	if theVersion is less than reqVers then
		display dialog "Blue Coconut requires Mac OS X " & reqVers & " or later" buttons {"Sorry"} default button "Sorry" giving up after 30 with icon 1
		return false
	end if
	
	return true
end checkVersion

on getReady()
	set status to contents of text field "Status" of window "mainWindow"
	if status = "Ready" or status starts with "Error:" or status = "Done" then
		set ready to true
	end if
end getReady

on getFiles(type)
	set sourceName to getiTunesSource(type)
	set trackList to getiTunesTracks(type)
	
	if sourceName  "" and trackList  {} then
		set beginning of queue to {shared:sourceName, tracks:trackList}
		
		if ready then
			getQueue()
		end if
		
	else
		if sourceName = "" then
			set longError to "iTunes source is not a shared library"
			set shortError to "not shared library"
		else if trackList = {} then
			set longError to "No iTunes selection found"
			set shortError to "no selection"
		else
			set longError to "Error"
			set shortError to "unknown"
		end if
		
		internalError(longError, shortError)
		
	end if
end getFiles

on getQueue()
	disableGet(false, false)
	setFeedback("start", "Gathering information")
	
	-- shift info off the list
	-- TODO fix the try/error block to work better
	try
		set info to first item of queue
		set queue to rest of queue
	on error
		return
	end try
	
	-- use this info
	set sourceName to shared of info
	set trackList to tracks of info
	
	try
		saveTemporaryFile(sourceName, trackList)
	on error
		internalError("Internal error" & return & return & "Couldn't write temporary file", "internal error")
		return
	end try
	
	do shell script "kill -USR1 " & pid
end getQueue

on reveal_iTunes()
	tell application "iTunes"
		set view of front browser window to playlist playlistName
		if not visible of front browser window then
			set visible of front browser window to true
		end if
		activate
	end tell
end reveal_iTunes

on reveal_Finder()
	tell application "Finder"
		set userMusic to the path to music folder from user domain
		open folder playlistName of folder userMusic
		activate
	end tell
end reveal_Finder

on showMainWindow()
	show window "MainWindow"
end showMainWindow

on showHTML()
	set resPath to resource path of main bundle
	set aliasPath to (POSIX file resPath as string) & ":index.html"
	tell application "Finder"
		open alias aliasPath
	end tell
end showHTML

-- UI feedback

on internalError(longError, shortError)
	set dialog to longError & return & return & "Select at least one track from a shared library"
	
	display dialog dialog buttons {"OK"} default button "OK" giving up after 30 with icon 0 attached to window "MainWindow"
	setFeedback("stop", "Error: " & shortError)
end internalError

on setFeedback(phase, message)
	tell window "mainWindow"
		if phase is "start" then
			start progress indicator "Spinner"
			set ready to false
		else if phase is "stop" then
			stop progress indicator "Spinner"
			set ready to true
		end if
		
	end tell
	setStatus(message)
end setFeedback

on setStatus(message)
	tell window "mainWindow"
		set the contents of text field "Status" to message
	end tell
end setStatus

on setInspector(message)
	tell window "mainWindow"
		set the contents of text field "Inspector" to ("iTunes status: " & message)
	end tell
end setInspector

on disableGet(sharedPlaying, sharedSelected)
	if not sharedPlaying then
		set downloadMenu to item 1 of (every menu item of sub menu of menu item id 2 of main menu whose name is "Download")
		set enabled of downloadMenu to false
		tell window "mainWindow"
			set enabled of button "Get" to false
		end tell
	end if
	if not sharedSelected then
		set downloadMenu to item 1 of (every menu item of sub menu of menu item id 2 of main menu whose name is "DownloadSelection")
		set enabled of downloadMenu to false
	end if
end disableGet

on enableGet(sharedPlaying, sharedSelected)
	if sharedPlaying then
		set downloadMenu to item 1 of (every menu item of sub menu of menu item id 2 of main menu whose name is "Download")
		set enabled of downloadMenu to true
		tell window "mainWindow"
			set enabled of button "Get" to true
		end tell
	end if
	if sharedSelected then
		set downloadMenu to item 1 of (every menu item of sub menu of menu item id 2 of main menu whose name is "DownloadSelection")
		set enabled of downloadMenu to true
	end if
end enableGet

-- iTunes interaction

on getiTunesStatus()
	set sharedPlaying to false
	set sharedSelected to false
	set message to ""
	
	tell application "System Events"
		if (creator type of processes) does not contain "hook" then
			set message to "not running"
		else
			
			tell application "iTunes"
				try
					if player state is playing then
						set message to "Playing"
						if my getiTunesSource("playing")  "" then
							set message to message & " a shared track"
							set sharedPlaying to true
						end if
					end if
					
					if my getiTunesSource("selection")  "" and selection  {} then
						if message = "" then
							set message to "Shared tracks selected"
						end if
						set sharedSelected to true
					end if
					
					if message = "" then
						set message to "Stopped"
					end if
					
				on error -- if we can't get the player state (eg long script)
					set message to "Not responding"
				end try -- don't worry if it can't get anything
			end tell -- iTunes
		end if -- is iTunes running?
	end tell -- System Events
	
	if visible of window "MainWindow" is true then
		setInspector(message)
	end if
	
	if (not sharedPlaying or not sharedSelected) then
		disableGet(sharedPlaying, sharedSelected)
	end if
	
	if ready and (sharedPlaying or sharedSelected) then
		enableGet(sharedPlaying, sharedSelected)
	end if
end getiTunesStatus

on getiTunesSource(type)
	set sourceName to ""
	
	tell application "iTunes"
		if type = "selection" then
			try -- avoid errors from iTMS windows, etc
				if kind of container of view of front window is shared library then
					set sourceName to name of container of view of front window as Unicode text
				end if
			on error
				log "error with kind of container of front window"
				
			end try
		end if
		
		if type = "playing" then
			try -- avoid errors from iTMS windows, etc
				if kind of container of container of current track is shared library then
					set sourceName to name of container of container of current track as Unicode text
				end if
			on error
				log "error with kind of container of current track"
			end try
		end if
	end tell
	
	return sourceName
end getiTunesSource

on getiTunesTracks(type)
	set trackList to {}
	
	tell application "iTunes"
		if type = "selection" then
			set theSelection to selection
			
			repeat with t in theSelection
				set trackRecord to {tit:name of t, art:artist of t, alb:album of t, did:database ID of t, siz:size of t, tra:track number of t, dis:disc number of t}
				set end of trackList to trackRecord
			end repeat
		end if
		
		if type = "playing" then
			set t to current track
			set trackRecord to {tit:name of t, art:artist of t, alb:album of t, did:database ID of t, siz:size of t, tra:track number of t, dis:disc number of t}
			set end of trackList to trackRecord
		end if
		
	end tell
	
	return trackList
	
end getiTunesTracks

on addAllToPlaylist()
	checkPlaylist()
	set trackList to readTemporaryFile()
	addToPlaylist(trackList)
end addAllToPlaylist

on checkPlaylist()
	tell application "iTunes"
		set thePlaylist to every playlist whose name is playlistName
		if thePlaylist = {} then
			set thePlaylist to make new playlist
			set the name of thePlaylist to playlistName
		end if
	end tell
end checkPlaylist

on addToPlaylist(trackList)
	tell application "iTunes"
		
		repeat with theTrack in trackList
			set theFile to (theTrack as POSIX file)
			add theFile to playlist playlistName
		end repeat
		
	end tell
end addToPlaylist

-- Perl communication

on saveTemporaryFile(sourceName, trackList)
	set filePath to getTemporaryFilePath()
	set ret to ASCII character 10
	set handle to open for access file filePath with write permission
	write ("sn:" & sourceName) to handle as class utf8
	write ret & ret to handle
	repeat with Track in trackList
		write "title:" & tit of Track & ret to handle as class utf8
		write "artist:" & art of Track & ret to handle as class utf8
		write "album:" & alb of Track & ret to handle as class utf8
		write "id:" & did of Track & ret to handle as class utf8
		write "size:" & siz of Track & ret to handle as class utf8
		write "track:" & tra of Track & ret to handle as class utf8
		write "disc:" & dis of Track & ret & ret to handle as class utf8
	end repeat
	
	close access handle
end saveTemporaryFile

on readTemporaryFile()
	set trackList to {}
	
	set filePath to getTemporaryFilePath()
	set theFile to open for access filePath
	set theData to read theFile as class utf8
	close access theFile
	
	set text item delimiters to ASCII character 10
	set trackList to trimList(text items of theData)
	return trackList
end readTemporaryFile

on trimList(oldList)
	set newList to {}
	
	set text item delimiters to ":"
	
	repeat with Entry in oldList
		if contents of Entry  "" and contents of Entry begins with "it:" then
			set theFile to the rest of the text items of Entry as Unicode text
			set end of newList to theFile
		end if
	end repeat
	
	return newList
end trimList

on do_mDNS(theSource)
	set logPath to POSIX path of ((path to "temp" from user domain as Unicode text) & "mdns.tmp")
	log logPath
	do shell script "mDNS -L \"" & theSource & "\" _daap._tcp > " & logPath & " 2>&1 & echo $!"
	do shell script "kill " & result
end do_mDNS

-- handy path stuff

on getLogFilePath()
	return (path to "dlib" from user domain as Unicode text) & ("Logs:BlueCoconut.log")
end getLogFilePath

on getTemporaryFilePath()
	return (path to "temp" from user domain as Unicode text) & ("bc.tmp")
end getTemporaryFilePath
