#!/usr/bin/perl
#
# BlueCoconut.pl - perl half of the Blue Coconut application

use strict;
use warnings;

use FindBin qw($Bin);
use File::Spec;

BEGIN {
  my $lib = File::Spec->catfile($Bin, "perl_lib");
  unshift @INC, $lib;
}

use Blue_Coconut::Rendezvous;
use Blue_Coconut::DAAP;
use Blue_Coconut::Guts;

my $daap   = Blue_Coconut::DAAP->new();
my $guts   = Blue_Coconut::Guts->new();
my $rv     = Blue_Coconut::Rendezvous->new;

# TODO timeout on the result of this
my $shares = $rv->get_shares();

$SIG{USR1} = \&handle;
$guts->reset_app_osa("Ready");

while (1) {
  sleep 5;
}

sub handle {
  get_files();
  $SIG{USR1} = \&handle;
  return;
}

sub get_files {
  eval {
    my $data   = $guts->read_from_applescript();
    my $tracks = $guts->get_tracks();
    
    $guts->set_status_osa("Locating server");
    my $info   = $guts->get_right_share($shares);
    
    $guts->set_status_osa("Connecting to server");
    $daap->connect($info);
    
    $guts->set_status_osa("Finding tracks on server");
    my $songs  = $daap->get_songs($tracks);
    
    my $count = 1;
    my $total = scalar(@{ $songs });
    
    unless (scalar(@{ $tracks }) == $total) {
      $daap = undef; # seems necessary to enable die
      $guts->die_osa("Cannot find requested tracks", 
                     "Could not find selection on server", 
                     "Selected tracks not found");
    }
    
    my $files = [];
    
    foreach my $song (@{ $songs }) {
      $guts->set_status_osa("Downloading song $count of $total");
        my $file = $daap->save_song($song);
      push @{ $files }, $file;
      $count++;
    }
    
    my $playlist = "Blue Coconut";
    
    $guts->set_status_osa(qq(Adding to $playlist playlist));
    $guts->add_files_to_playlist($files);
    
    $guts->reset_app_osa;
  }
}
