# Blue Coconut version 1.0, Copyright 2004 Paul Mison.

use strict;
use warnings;

use Encode;
package Blue_Coconut::Guts;

use File::Path;
use File::Spec;

sub new {
    # Typical Perl constructor
    # See 'perltoot' for details
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {
        'wc' => undef,
    };
    bless ($self, $class);

    return $self;
}

sub add_files_to_playlist {
  my $self  = shift;
  my $files = shift;

  my $done = $self->write_to_applescript($files);
  if ($done) {
    $self->add_files_to_playlist_osa();
    $self->remove_temp_file();
  } else {
    $self->die_osa('Warning: iTunes not updated',
                   'Tracks downloaded successfully',
                   'Done');
  }
}

sub read_from_applescript {
  my $self = shift;
  
  my ($share, $tracks);
  my $track = {};
  
  my $path = $self->{path} || $self->get_temp_dir_osa();
  my $file = File::Spec->catfile($path, 'bc.tmp');
  $self->{path} = $path;
  $self->{file} = $file;
  
  open (HANDLE, $file) or $self->die_osa("Internal error", "Could not find temporary file", "Error: internal file error");
  binmode HANDLE, 'utf8';

  while (<HANDLE>) {
    chomp;

    if ($_ =~ s/^sn://) {
      $self->{source} = $_;
    }
    
    if ($_ eq '') {
      push @{ $tracks }, $track if (%{ $track });
      $track = {};
    }

    if ($_ =~ m/^(title|artist|album|id|size|track|disc)/) {
      my ($key, $value) = split(/:/, $_, 2);
      $track->{$key} = $value;
    }
  }
  $self->{tracks} = $tracks;
  close (HANDLE);

  if (scalar (@{ $tracks }) == 0) {
    $self->die_osa("Internal error",
                   "No tracks listed in temporary file",
                   "Error: internal file error");
  }
  
  $self->remove_temp_file();
  return;
}

sub write_to_applescript {
  my $self = shift;
  my $list = shift;
  
  my $file = $self->{file} || File::Spec->catfile($self->get_temp_dir_osa(), "bc.tmp");
  open (HANDLE, ">$file") or return undef;
  binmode HANDLE, 'utf8';

  foreach my $song (@{ $list }) {
    print HANDLE "it:$song\n";
  }
  close (HANDLE);
  return 1;
}

sub remove_temp_file {
  my $self = shift;
  my $file = $self->{file} or return;

  unlink $file;
}
  
sub get_right_share {
  my $self   = shift;
  my $shares = shift;

  my $share  = $self->{source};
  
  foreach my $daap (keys %{ $shares }) {
    if ($daap eq $share) {
      return $shares->{$daap};
    }
  }

  $self->die_osa("Could not find shared library", "Source library not found via Rendezvous", "Error: shared library not found");
}

sub get_tracks {
  my $self = shift;
  return $self->{tracks};
}

sub get_save_dir {
  my $self     = shift;

  return $self->{save_dir} if $self->{save_dir};

  # TODO configure name, location
  my $playlist = "Blue Coconut"; # TODO rationalise this variable
  my $dir = File::Spec->catfile($ENV{HOME}, "Music", $playlist);
  
  $self->{save_dir} = $dir;
  return $self->{save_dir};
}

sub get_log_dir {
  my $self   = shift;

  return File::Spec->catfile($ENV{HOME}, "Library", "Logs");
}

sub record_filename {
  my $self = shift;

    
}

sub get_temp_dir_osa {
  my $self = shift;
  
  my $command = 'get POSIX path of (path to "temp" from user domain as text)';
  my $path    = `osascript -e '$command'`;
  chomp $path;
  return $path;
}

sub set_status_osa {
  my $self = shift;
  my $text = shift;

  my $script = 'tell application "Blue Coconut" to set the contents of text field "Status" of the window "mainWindow" to "';
  `osascript -e '$script$text"'`;
}

sub add_files_to_playlist_osa {
  my $self = shift;
  

  my $script = <<EOF;
tell application "Blue Coconut"
	set s to (get script of button 1 of window 1)
  tell s to addAllToPlaylist()
end tell
EOF

  `osascript -e '$script'`;
}

sub die_osa {
  my $self = shift;
  my ($intro, $body, $short) = @_;

  my $script = <<EOF;
tell application "Blue Coconut"
  display dialog "$intro" & return & return & "$body" buttons {"OK"} default button "OK" giving up after 30 with icon 0 attached to window "MainWindow"
end tell
EOF

  `osascript -e '$script'`;

  $self->reset_app_osa($short);
  die "$intro ($body)";
}

sub reset_app_osa {
  my $self = shift;
  my $text = shift || "Done";
  
  my $script = <<EOF;
tell application "Blue Coconut"
	tell window "mainWindow"
		tell progress indicator "Spinner" to stop
		-- set enabled of button "Get" to true
		set the contents of text field "Status" to "$text"
	end tell
end tell
EOF
 
  `osascript -e '$script'`;
}

1;
